summaryrefslogtreecommitdiff
path: root/gnu/java
diff options
context:
space:
mode:
authorMark Wielaard <mark@klomp.org>2003-12-25 16:36:55 +0000
committerMark Wielaard <mark@klomp.org>2003-12-25 16:36:55 +0000
commit3ae8e95d01cb8ae899c0f859f1c1a66dd34ea031 (patch)
treec4f2cc498267684aa94960a3c8adc0f633e073c9 /gnu/java
parent58f43a6d4734f4ea671c2e69955955dd6239b6d2 (diff)
downloadclasspath-3ae8e95d01cb8ae899c0f859f1c1a66dd34ea031.tar.gz
2003-12-25 Guilhem Lavaux <guilhem@kaffe.org>
* gnu/java/net/URLParseError.java: New file. * gnu/java/net/protocol/jar/Handler.java (parseURL): Throw URLParseError if needed, fix '/' handling. * java/net/URL.java (URL): Catch URLParseError and transform it into a MalformedURLException. * gnu/java/net/Makefile.am (EXTRA_DIST): Add URLParseError.java.
Diffstat (limited to 'gnu/java')
-rw-r--r--gnu/java/net/Makefile.am4
-rw-r--r--gnu/java/net/URLParseError.java57
-rw-r--r--gnu/java/net/protocol/jar/Handler.java30
3 files changed, 82 insertions, 9 deletions
diff --git a/gnu/java/net/Makefile.am b/gnu/java/net/Makefile.am
index d858c82aa..e96118aa2 100644
--- a/gnu/java/net/Makefile.am
+++ b/gnu/java/net/Makefile.am
@@ -5,5 +5,5 @@ SUBDIRS = content protocol
EXTRA_DIST = \
HeaderFieldHelper.java \
PlainDatagramSocketImpl.java \
-PlainSocketImpl.java
-
+PlainSocketImpl.java \
+URLParseError.java
diff --git a/gnu/java/net/URLParseError.java b/gnu/java/net/URLParseError.java
new file mode 100644
index 000000000..95eed65dd
--- /dev/null
+++ b/gnu/java/net/URLParseError.java
@@ -0,0 +1,57 @@
+/* URLParseError.java -- Helps bypassing the exception limitation for
+ URLStreamHandler.parseURL().
+ Copyright (C) 2003 Free Software Foundation, Inc.
+
+This file is part of GNU Classpath.
+
+GNU Classpath is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
+
+GNU Classpath is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Classpath; see the file COPYING. If not, write to the
+Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+02111-1307 USA.
+
+Linking this library statically or dynamically with other modules is
+making a combined work based on this library. Thus, the terms and
+conditions of the GNU General Public License cover the whole
+combination.
+
+As a special exception, the copyright holders of this library give you
+permission to link this library with independent modules to produce an
+executable, regardless of the license terms of these independent
+modules, and to copy and distribute the resulting executable under
+terms of your choice, provided that you also meet, for each linked
+independent module, the terms and conditions of the license of that
+module. An independent module is a module which is not derived from
+or based on this library. If you modify this library, you may extend
+this exception to your version of the library, but you are not
+obligated to do so. If you do not wish to do so, delete this
+exception statement from your version. */
+
+package gnu.java.net;
+
+/**
+ * This class helps the people writing protocols to report URL parse
+ * errors in parseUrl as this method cannot report other exceptions
+ * than Errors.
+ *
+ * The main drawback is that it uses the Error mechanism which should not
+ * be used for that type of error reporting.
+ *
+ * @author Guilhem Lavaux <guilhem@kaffe.org>
+ */
+public class URLParseError extends Error
+{
+ public URLParseError(String msg)
+ {
+ super(msg);
+ }
+}
diff --git a/gnu/java/net/protocol/jar/Handler.java b/gnu/java/net/protocol/jar/Handler.java
index 0be9991df..d1235032c 100644
--- a/gnu/java/net/protocol/jar/Handler.java
+++ b/gnu/java/net/protocol/jar/Handler.java
@@ -38,7 +38,9 @@ exception statement from your version. */
package gnu.java.net.protocol.jar;
+import gnu.java.net.URLParseError;
import java.io.IOException;
+import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
@@ -70,7 +72,7 @@ public class Handler extends URLStreamHandler
}
/**
* This method overrides URLStreamHandler's for parsing url of protocol "file"
*
* @param url The URL object in which to store the results
* @param url_string The String-ized URL to parse
@@ -91,10 +93,11 @@ public class Handler extends URLStreamHandler
if (url_string.startsWith("/"))
{ //url string is an absolute path
int idx = file.lastIndexOf ("!/");
- if (idx == -1) //context path is weird
- file = file + "!" + url_string;
- else
- file = file.substring (0, idx + 1) + url_string;
+
+ if (idx < 0)
+ throw new URLParseError("no !/ in spec");
+
+ file = file.substring (0, idx + 1) + url_string;
}
else
{
@@ -108,7 +111,7 @@ public class Handler extends URLStreamHandler
// according to Java API Documentation, here is a little different
// with URLStreamHandler.parseURL
// but JDK seems doesn't handle it well
- file = file + "/" + url_string;
+ file = file.substring(0, idx + 1) + url_string;
}
setURL (url, "jar", url.getHost(), url.getPort(), file, null);
@@ -125,9 +128,22 @@ public class Handler extends URLStreamHandler
// Skip remains of protocol
url_string = url_string.substring (start, end);
+
+ int jar_stop;
+ if ((jar_stop = url_string.indexOf("!/")) < 0)
+ throw new URLParseError("no !/ in spec");
+
+ try
+ {
+ new URL(url_string.substring (0, jar_stop));
+ }
+ catch (MalformedURLException e)
+ {
+ throw new URLParseError("invalid inner URL: " + e.getMessage());
+ }
if (!url.getProtocol().equals ("jar") )
- return;
+ throw new URLParseError("unexpected protocol " + url.getProtocol());
setURL (url, "jar", url.getHost(), url.getPort(), url_string, null);
}
@@ -70,7 +72,7 @@ public class Handler extends URLStreamHandler
}
/**
- * This method overrides URLStreamHandler's for parsing url of protocol "file"
+ * This method overrides URLStreamHandler's for parsing url of protocol "jar"
*
* @param url The URL object in which to store the results